import React, { PropTypes } from 'react'
import Markdown from 'react-remarkable'
import styled, { css } from 'styled-components'
import Box from './Box'
import theme from './theme'

const getMaxLineWidth = (maxLineWidth) => {
  if (typeof maxLineWidth === 'boolean') {
    if (maxLineWidth) return `max-width: 40em;`
    return ''
  }
  return `max-width: ${maxLineWidth};`
}

const getTextStyle = ({ fontSize, fontWeight, lineHeight, letterSpacing }) => `
  font-size: ${fontSize};
  font-weight: ${fontWeight};
  line-height: ${lineHeight};
  letter-spacing: ${letterSpacing};
`

const styles = ({ indentLists, theme, maxLineWidth }) => css`
  html, body, div, span, applet, object, iframe,
  h1, h2, h3, h4, h5, h6, p, blockquote, pre,
  a, abbr, acronym, address, big, cite, code,
  del, dfn, em, img, ins, kbd, q, s, samp,
  small, strike, strong, sub, sup, tt, var,
  b, u, i, center,
  dl, dt, dd, ol, ul, li,
  fieldset, form, label, legend,
  table, caption, tbody, tfoot, thead, tr, th, td,
  article, aside, canvas, details, embed,
  figure, figcaption, footer, header, hgroup,
  menu, nav, output, ruby, section, summary,
  time, mark, audio, video {
    margin: 0;
    padding: 0;
    border: 0;
    font-size: 100%;
    font: inherit;
    vertical-align: baseline;
  }

  ol, ul {
    list-style: none;
  }
  blockquote, q {
    quotes: none;
  }

  blockquote:before, blockquote:after,
  q:before, q:after {
    content: '';
    content: none;
  }

  table {
    border-collapse: collapse;
    border-spacing: 0;
  }

  input,
  textarea {
    border: none;
    outline: none;
  }
  
  a {
    color: ${theme.colors.primary}
  }

  h1 {
    ${getTextStyle(theme.textStyles.display1)}
    margin-top: 1em;
    margin-bottom: 1em;
  }

  h2 {
    ${getTextStyle(theme.textStyles.display2)}
    margin-top: 1em;
    margin-bottom: 1em;
  }

  h3 {
    ${getTextStyle(theme.textStyles.display3)}
    margin-top: 1em;
    margin-bottom: 1em;
  }

  h4, h5, h6 {
    ${getTextStyle(theme.textStyles.display4)}
    margin-top: 1em;
    margin-bottom: 1em;
  }

  p, ul, ol {
    ${getTextStyle(theme.textStyles.body1)}
    ${getMaxLineWidth(maxLineWidth)}
    margin: 1em 0;
  }

  ${indentLists ? `
    ul,
    ol {
      margin-left: 1.15em;
    }
  ` : ''}

  ul {
    list-style: disc;
    list-style-position: outside;
  }

  ol {
    list-style: decimal;
  }

  em {
    font-style: italic;
  }

  strong {
    font-weight: 500;
  }

  p > code,
  li > code,
  table code {
    ${getTextStyle(theme.textStyles.body1)}
    display: inline-block;
    background-color: ${theme.colors.tint[0]};
    box-shadow: inset 0 0 0 1px ${theme.colors.tint[1]};
    padding: 0 0.5em;
    border-radius: ${theme.borderRadius[0]};

    &:before,
    &:after {
      content: '\\00a0';
      letter-spacing: -0.2em;
    }
  }

  pre {
    margin: 1em 0;
    overflow-x: auto;
    text-align: left;
    display: block;
    white-space: pre;
    background-color: ${theme.colors.tint[0]};
    box-shadow: inset 0 0 0 1px ${theme.colors.tint[1]};
    padding: ${theme.space[3]};
    border-radius: ${theme.borderRadius[0]};

    & code {
      ${getTextStyle(theme.textStyles.mono)}
      padding: 0;
      background: none;

      &:before,
      &:after {
        content: none;
        letter-spacing: 0;
      }
    }
  }

  hr {
    background: ${theme.colors.tint[1]};
    border: 0;1
    height: 1px;
    margin: 30px 0;
  }

  table {
    width: 100%;
    margin: 1em 0;
  }

  thead {
    border-bottom: 1px solid ${theme.colors.tint[1]};
    background-color: ${theme.colors.tint[0]};
    position: relative;
    text-align: left;
    padding: 0 20px;
    box-sizing: content-box;
  }

  tbody tr {
    border-bottom: 1px solid ${theme.colors.tint[1]};
    background: ${theme.colors.white[0]};
  }

  td, th {
    padding: 0.8em;
  }
`

const parseProps = ({ children, ...props }) => {
  if (typeof children === 'string') {
    return { children: React.createElement(Markdown, { source: children }) , ...props }
  }
  return { children, ...props }
}

const Content = styled(({ markdown, indentLists, maxLineWidth, ...props }) =>
  React.createElement(Box, parseProps(props)))`${styles}`

Content.propTypes = {
  ...Box.propTypes,
  markdown: PropTypes.bool,
  indentLists: PropTypes.bool,
  maxLineWidth: PropTypes.oneOfType([
    PropTypes.bool,
    PropTypes.string
  ])
}

Content.defaultProps = {
  theme,
  indentLists: true,
  maxLineWidth: true
}

Content.displayName = 'Content'

export default Content
